package me.mrletsplay.advancedinfo;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.mojang.brigadier.CommandDispatcher;

import me.mrletsplay.advancedinfo.command.VillagerInfoCommand;
import me.mrletsplay.advancedinfo.command.WeatherForecastCommand;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.minecraft.command.CommandRegistryAccess;
import net.minecraft.entity.Entity;
import net.minecraft.server.command.CommandManager.RegistrationEnvironment;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.util.math.Box;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

public class AdvancedInfoMod implements ModInitializer {

	// This logger is used to write text to the console and the log file.
	// It is considered best practice to use your mod id as the logger's name.
	// That way, it's clear which mod wrote info, warnings, and errors.
	public static final Logger LOGGER = LoggerFactory.getLogger("advanced-info");

	@Override
	public void onInitialize() {
		// This code runs as soon as Minecraft is in a mod-load-ready state.
		// However, some things (like resources) may still be uninitialized.
		// Proceed with mild caution.

		LOGGER.info("Hello Fabric world!");

		CommandRegistrationCallback.EVENT.register(new CommandRegistrationCallback() {

			@Override
			public void register(CommandDispatcher<ServerCommandSource> dispatcher, CommandRegistryAccess registryAccess, RegistrationEnvironment environment) {
				dispatcher.register(new VillagerInfoCommand().create());
				dispatcher.register(new WeatherForecastCommand().create());
			}
		});
	}

	public static Entity raycastEntity(World world, Vec3d from, Vec3d to) {
		Box b = new Box(from, to);
		List<Entity> entities = world.getOtherEntities(null, b);

		Entity closest = null;
		double closestDistance = Double.MAX_VALUE;
		for(Entity e : entities) {
			Vec3d point = e.getBoundingBox().raycast(from, to).orElse(null);
			if(point == null) continue;

			double distance = point.distanceTo(from);
			if(closest == null || distance < closestDistance) {
				closest = e;
				closestDistance = distance;
			}
		}

		return closest;
	}

}